@@ -1,4 +1,4 @@
-/* Mixmaster version 3.0  --  (C) 1999 - 2006 Anonymizer Inc. and others.
+/* Mixmaster version 3.1 --  (C) 1999 - 2016 Anonymizer Inc. and others.
 
    Mixmaster may be redistributed and modified under certain conditions.
    This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF
@@ -6,7 +6,7 @@
    details.
 
    Prepare messages for remailer chain
-   $Id: chain.c 934 2006-06-24 13:40:39Z rabbi $ */
+   $Id$ */
 
 
 #include "mix3.h"
@@ -103,6 +103,10 @@ void parse_badchains(int badchains[MAXREM][MAXREM], char *file, char *startindic
 int chain_select(int hop[], char *chainstr, int maxrem, REMAILER *remailer,
 		 int type, BUFFER *feedback)
 {
+/* hop[] is returned containing the chain as integers (0 means random like *)
+ * chainstr is the input desired chain such as *,*,*,*
+ * remailer is an input list of remailer details (see mix2_rlist())
+ */
   int len = 0;
   int i, j, k;
   BUFFER *chain, *selected, *addr;
