@@ -1,4 +1,4 @@
-/* Mixmaster version 3.0  --  (C) 1999 - 2006 Anonymizer Inc. and others.
+/* Mixmaster version 3.1 --  (C) 1999 - 2016 Anonymizer Inc. and others.
 
    Mixmaster may be redistributed and modified under certain conditions.
    This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF
@@ -6,7 +6,7 @@
    details.
 
    Interface to cryptographic library
-   $Id: crypto.c 934 2006-06-24 13:40:39Z rabbi $ */
+   $Id$ */
 
 
 #include "mix3.h"
@@ -58,6 +58,59 @@ int digest_sha1(BUFFER *b, BUFFER *md)
   return (digestmem_sha1(b->data, b->length, md));
 }
 
+static int digestmem_sha256(byte *b, int n, BUFFER *md)
+{
+  byte m[SHA256_DIGEST_LENGTH];
+
+  SHA256((unsigned char *)b, (unsigned long)n, (unsigned char *)m);
+  buf_reset(md);
+  buf_append(md, m, SHA256_DIGEST_LENGTH);
+  return (0);
+}
+
+int digest_sha256(BUFFER *b, BUFFER *md)
+{
+  return (digestmem_sha256(b->data, b->length, md));
+}
+
+int isdigest_sha256(BUFFER *b, BUFFER *md)
+{
+  int ret;
+  BUFFER *newmd;
+
+  newmd = buf_new();
+  digest_sha256(b, newmd);
+  ret = buf_eq(md, newmd);
+  buf_free(newmd);
+  return (ret);
+}
+
+int hmac_sha256(BUFFER *b, BUFFER *hk, BUFFER *md) {
+  BUFFER *pad, *middle;
+  int ret;
+  int ipadbyte=0x36;
+  int opadbyte=0x5C;
+  char *i;
+
+  if (64 != hk->length) return -1;
+  pad=buf_new();
+  middle=buf_new();
+  buf_cat(pad,hk);
+  for (i=pad->data; i < (pad->data + pad->length); i++)
+     *i ^= ipadbyte;
+  buf_cat(pad,b);
+  ret=digest_sha256(pad,middle);
+
+  buf_set(pad,hk);
+  for (i=pad->data; i < (pad->data + pad->length); i++)
+     *i ^= opadbyte;
+  buf_cat(pad,middle);
+  if (!ret) ret=digest_sha256(pad,md);
+  buf_free(pad);
+  buf_free(middle);
+  return (ret);
+}
+
 static int digestmem_rmd160(byte *b, int n, BUFFER *md)
 {
   byte m[RIPEMD160_DIGEST_LENGTH];
@@ -73,7 +126,7 @@ int digest_rmd160(BUFFER *b, BUFFER *md)
   return (digestmem_rmd160(b->data, b->length, md));
 }
 
-#define MAX_RSA_MODULUS_LEN 128
+#define MAX_RSA_MODULUS_LEN 4096
 
 static int read_seckey(BUFFER *buf, SECKEY *key, const byte id[])
 {
@@ -89,8 +142,10 @@ static int read_seckey(BUFFER *buf, SECKEY *key, const byte id[])
   plen = (len + 1) / 2;
 
   /* due to encryption, buffer size is multiple of 8 */
-  if (3 * len + 5 * plen + 8 < buf->length || 3 * len + 5 * plen > buf->length)
+  /* secret key structure storage is 3 items of full modulus length (n,e,d) and 5 of half that length  */
+  if (3 * len + 5 * plen + 8 < buf->length || 3 * len + 5 * plen > buf->length) {
     return (-1);
+  }
 
   ptr = buf->data + 2;
 
@@ -161,69 +216,84 @@ static int read_pubkey(BUFFER *buf, PUBKEY *key, const byte id[])
 
 static int write_seckey(BUFFER *sk, SECKEY *key, byte keyid[])
 {
-  byte l[128];
+  byte l[512];
   int n;
   BUFFER *b, *temp;
+  int nn;   /* modulus length in bytes */
+
+  n = BN_bn2bin(key->n, l); /* writing modulus */
+  assert((n==128) || (n==256) || (n==384) || (n==512));
+  switch(n) {
+      case 128:
+      case 256:
+      case 384:
+      case 512:
+        nn=n;
+        break;
+      default:
+        return -1;
+        break;
+  }
 
   b = buf_new();
   temp = buf_new();
 
-  n = BN_bn2bin(key->n, l);
-  assert(n <= 128);
-  if (n < 128)
-    buf_appendzero(b, 128 - n);
+  if (n < nn)
+    buf_appendzero(b, nn - n);
   buf_append(b, l, n);
 
-  n = BN_bn2bin(key->e, l);
-  assert(n <= 128);
-  if (n < 128)
-    buf_appendzero(b, 128 - n);
+  n = BN_bn2bin(key->e, l);   /* writing e */
+  assert(n <= nn);
+  if (n < nn)
+    buf_appendzero(b, nn - n);
   buf_append(b, l, n);
 
   digest_md5(b, temp);
   memcpy(keyid, temp->data, 16);
 
   buf_appendc(sk, 0);
-  buf_appendc(sk, 4);
+  buf_appendc(sk, nn/32);  /* nn of 128 bytes = 1024-bits = (4*256)+0 so store 4 */
   buf_cat(sk, b);
 
-  n = BN_bn2bin(key->d, l);
-  assert(n <= 128);
-  if (n < 128)
-    buf_appendzero(sk, 128 - n);
+  n = BN_bn2bin(key->d, l);   /* writing d */
+  assert(n <= nn);
+  if (n < nn)
+    buf_appendzero(sk, nn - n);
   buf_append(sk, l, n);
 
+  nn /= 2;  /* now store the smaller pieces */
+
   n = BN_bn2bin(key->p, l);
-  assert(n <= 64);
-  if (n < 64)
-    buf_appendzero(sk, 64 - n);
+  assert(n <= nn);
+  if (n < nn)
+    buf_appendzero(sk, nn - n);
   buf_append(sk, l, n);
 
   n = BN_bn2bin(key->q, l);
-  assert(n <= 64);
-  if (n < 64)
-    buf_appendzero(sk, 64 - n);
+  assert(n <= nn);
+  if (n < nn)
+    buf_appendzero(sk, nn - n);
   buf_append(sk, l, n);
 
   n = BN_bn2bin(key->dmp1, l);
-  assert(n <= 64);
-  if (n < 64)
-    buf_appendzero(sk, 64 - n);
+  assert(n <= nn);
+  if (n < nn)
+    buf_appendzero(sk, nn - n);
   buf_append(sk, l, n);
 
   n = BN_bn2bin(key->dmq1, l);
-  assert(n <= 64);
-  if (n < 64)
-    buf_appendzero(sk, 64 - n);
+  assert(n <= nn);
+  if (n < nn)
+    buf_appendzero(sk, nn - n);
   buf_append(sk, l, n);
 
   n = BN_bn2bin(key->iqmp, l);
-  assert(n <= 64);
-  if (n < 64)
-    buf_appendzero(sk, 64 - n);
+  assert(n <= nn);
+  if (n < nn)
+    buf_appendzero(sk, nn - n);
   buf_append(sk, l, n);
 
-  buf_pad(sk, 712);		/* encrypt needs a block size multiple of 8 */
+  buf_pad(sk, 8*((sk->length+7)/8));		/* encrypt needs a block size multiple of 8 */
 
   buf_free(temp);
   buf_free(b);
@@ -232,20 +302,32 @@ static int write_seckey(BUFFER *sk, SECKEY *key, byte keyid[])
 
 static int write_pubkey(BUFFER *pk, PUBKEY *key, byte keyid[])
 {
-  byte l[128];
+  byte l[512];
   int n;
+  int nn;
 
-  buf_appendc(pk, 0);
-  buf_appendc(pk, 4);
   n = BN_bn2bin(key->n, l);
-  assert(n <= 128);
-  if (n < 128)
-    buf_appendzero(pk, 128 - n);
+  switch(n) {
+      case 128:
+      case 256:
+      case 384:
+      case 512:
+        nn=n;
+        break;
+      default:
+        return -1;
+        break;
+  }
+  buf_appendc(pk, 0);
+  buf_appendc(pk, nn/32);
+  assert(n <= nn);
+  if (n < nn)
+    buf_appendzero(pk, nn - n);
   buf_append(pk, l, n);
   n = BN_bn2bin(key->e, l);
-  assert(n <= 128);
-  if (n < 128)
-    buf_appendzero(pk, 128 - n);
+  assert(n <= nn);
+  if (n < nn)
+    buf_appendzero(pk, nn - n);
   buf_append(pk, l, n);
   return (0);
 }
@@ -285,7 +367,7 @@ int check_seckey(BUFFER *buf, const byte id[])
   return (ret);
 }
 
-int v2createkey(void)
+int v2createkey(long int lifeindays,long int keysize)
 {
   RSA *k;
   BUFFER *b, *ek, *iv;
@@ -298,8 +380,8 @@ int v2createkey(void)
   ek = buf_new();
   iv = buf_new();
 
-  errlog(NOTICE, "Generating RSA key.\n");
-  k = RSA_generate_key(1024, 65537, NULL, NULL);
+  errlog(NOTICE, "Generating RSA key of size %ld.\n", keysize);
+  k = RSA_generate_key(keysize, 65537, NULL, NULL);
   err = write_seckey(b, k, keyid);
   RSA_free(k);
   if (err == 0) {
@@ -310,8 +392,13 @@ int v2createkey(void)
       gt = gmtime(&now);
       strftime(line, LINELEN, "%Y-%m-%d", gt);
       fprintf(f, "%s\nCreated: %s\n", begin_key, line);
+/* lifeindays comes from the command line and may override
+   the default of KEYLIFETIME (in seconds). */
       if (KEYLIFETIME) {
-	now += KEYLIFETIME;
+        if ((lifeindays>0) && (lifeindays*SECONDSPERDAY<KEYLIFETIME))
+	    now += lifeindays*SECONDSPERDAY;
+        else
+	    now += KEYLIFETIME;
 	gt = gmtime(&now);
 	strftime(line, LINELEN, "%Y-%m-%d", gt);
 	fprintf(f, "Expires: %s\n", line);
@@ -454,6 +541,10 @@ int buf_castcrypt(BUFFER *buf, BUFFER *key, BUFFER *iv, int enc)
   return (0);
 }
 
+#ifndef USE_AES
+#error AES is compulsory in this version
+#endif
+
 #ifdef USE_AES
 int buf_aescrypt(BUFFER *buf, BUFFER *key, BUFFER *iv, int enc)
 {
@@ -468,7 +559,74 @@ int buf_aescrypt(BUFFER *buf, BUFFER *key, BUFFER *iv, int enc)
   AES_set_encrypt_key(key->data, key->length<<3, &ks);
   AES_cfb128_encrypt(buf->data, buf->data, buf->length, &ks, iv->data, &n,
 		     enc == ENCRYPT ? AES_ENCRYPT : AES_DECRYPT);
-  return (0);
+  return (n);
+}
+
+// AES CTR mode is identical for encrypt/decrypt
+// from https://github.com/crooks/mixmaster4096/commit/ce2756b0858f782b60efb579d67b7baf0572e19e
+int buf_aes_ctr128(BUFFER *buf, BUFFER *key, BUFFER *iv)
+{
+unsigned int n = 0;
+unsigned char ecount[AES_BLOCK_SIZE];
+AES_KEY ks;
+memset(ecount, 0, sizeof(ecount));
+
+if (key == NULL || key->length == 0)
+return (-1);
+
+assert((key->length == 16 || key->length == 24 || key->length == 32) && iv->length == 16);
+AES_set_encrypt_key(key->data, key->length<<3, &ks);
+AES_ctr128_encrypt(buf->data, buf->data, buf->length, &ks, iv->data, ecount, &n);
+return (n);
+}
+
+int derive_aes_keys(BUFFER *aes_pre_key, BUFFER *hkey, BUFFER *aes_header_key,
+ BUFFER *aes_body_key, BUFFER *aes_tte_key, BUFFER *aes_iv, BUFFER *aes_body_iv , BUFFER *aes_header_iv)
+{
+/* Use the aes_pre_key and hkey data (known at both ends) to derive
+ * 3 AES keys and an IV.
+ */
+
+BUFFER *tempmac, *tempiv;
+tempmac=buf_new();
+tempiv=buf_new();
+
+buf_reset(aes_header_key);
+buf_sets(tempmac, "header-crypt");
+buf_cat(tempmac, aes_pre_key);
+hmac_sha256(tempmac, hkey, aes_header_key);
+
+buf_reset(aes_body_key);
+buf_sets(tempmac, "body-crypt");
+buf_cat(tempmac, aes_pre_key);
+hmac_sha256(tempmac, hkey, aes_body_key);
+
+buf_reset(aes_tte_key);
+buf_sets(tempmac, "tte-crypt");
+buf_cat(tempmac, aes_pre_key);
+hmac_sha256(tempmac, hkey, aes_tte_key);
+
+buf_reset(aes_iv);
+buf_sets(tempmac, "aes-iv");
+buf_cat(tempmac, aes_pre_key);
+hmac_sha256(tempmac, hkey, tempiv);
+buf_append(aes_iv, tempiv->data, 16);
+
+buf_reset(aes_body_iv);
+buf_sets(tempmac, "aes-body-iv");
+buf_cat(tempmac, aes_pre_key);
+hmac_sha256(tempmac, hkey, tempiv);
+buf_append(aes_body_iv, tempiv->data, 16);
+
+buf_reset(aes_header_iv);
+buf_sets(tempmac, "aes-header-iv");
+buf_cat(tempmac, aes_pre_key);
+hmac_sha256(tempmac, hkey, tempiv);
+buf_append(aes_header_iv, tempiv->data, 16);
+
+buf_free(tempmac);
+buf_free(tempiv);
+return 0;
 }
 #endif /* USE_AES */
 
