@@ -1,4 +1,4 @@
-/* Mixmaster version 3.0  --  (C) 1999 - 2006 Anonymizer Inc. and others.
+/* Mixmaster version 3.1 --  (C) 1999 - 2016 Anonymizer Inc. and others.
 
    Mixmaster may be redistributed and modified under certain conditions.
    This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF
@@ -6,7 +6,7 @@
    details.
 
    Socket-based mail transport services
-   $Id: mail.c 934 2006-06-24 13:40:39Z rabbi $ */
+   $Id$ */
 
 
 #include "mix3.h"
@@ -425,7 +425,7 @@ SOCKET smtp_open(void)
 
 #ifdef USE_SOCK
   if (SMTPRELAY[0] != '\0')
-    s = opensocket(SMTPRELAY, 25);
+    s = opensocket(SMTPRELAY, SMTPPORT);
   if (s != INVALID_SOCKET) {
     line = buf_new();
     sock_getsmtp(s, line);
