@@ -1,4 +1,4 @@
-/* Mixmaster version 3.0  --  (C) 1999 - 2006 Anonymizer Inc. and others.
+/* Mixmaster version 3.1 --  (C) 1999 - 2016 Anonymizer Inc. and others.
 
    Mixmaster may be redistributed and modified under certain conditions.
    This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF
@@ -6,7 +6,7 @@
    details.
 
    Command-line based frontend
-   $Id: main.c 937 2006-06-24 15:52:20Z colin $ */
+   $Id$ */
 
 
 #include "mix3.h"
@@ -45,6 +45,8 @@ int main(int argc, char *argv[])
   int daemon = 0, type_list = 0, nodetach = 0;
   int update_stats = 0, update_pingerlist = 0;
   int never_ask_for_passphrase = 0;
+  long int lifeindays=0;
+  long int keysize=4096;
 
 #ifdef USE_SOCK
   int pop3 = 0;
@@ -75,6 +77,7 @@ int main(int argc, char *argv[])
   field = buf_new();
   content = buf_new();
   statssrc = buf_new();
+  mix_global_verbose=0;
 
 #ifdef USE_NCURSES
   if (argc == 1) {
@@ -100,9 +103,10 @@ int main(int argc, char *argv[])
 	  version = 1, deflt = 0;
 	else if (streq(p, "about"))
 	  about = 1, deflt = 0;
-	else if (streq(p, "verbose"))
+	else if (streq(p, "verbose")) {
 	  verbose = 1;
-	else if (streq(p, "type-list"))
+	  mix_global_verbose = 1;
+	} else if (streq(p, "type-list"))
 	  type_list = 1;
 	else if (streq(p, "dummy"))
 	  send = MSG_NULL, deflt = 0;
@@ -153,6 +157,23 @@ int main(int argc, char *argv[])
 	    deflt = 0;
 	    fprintf(stderr, "%s: No current stats source --%s\n", argv[0], p);
 	  }
+	} else if (strleft(p, "lifetime") && p[strlen("lifetime")] == '=') {
+            lifeindays=strtol(p + strlen("lifetime") + 1,NULL,10);
+            if (lifeindays<0)
+                lifeindays=0;
+	} else if (strleft(p, "size") && p[strlen("size")] == '=') {
+            switch(keysize) {
+             case 1024:
+             case 2048:
+             case 3072:
+             case 4096:
+               /* only certain permitted sizes */
+               keysize=strtol(p + strlen("size") + 1,NULL,10);
+               break;
+             default:
+               /* ignoring invalid size seen on comand-line */
+               break;
+            }
 	} else if (strleft(p, "update-stats") && p[strlen("update-stats")] == '=') {
 	  buf_clear(statssrc);
 	  buf_appendf(statssrc, "%s", (p + strlen("update-stats") + 1));
@@ -240,6 +261,7 @@ int main(int argc, char *argv[])
 	    break;
 	  case 'v':
 	    verbose = 1;
+	    mix_global_verbose = 1;
 	    break;
 	  case 'h':
 	    help = 1, deflt = 0;
@@ -432,6 +454,7 @@ int main(int argc, char *argv[])
 -h, --help                        summary of command line options\n\
 -V, --version                     print version information\n\
     --about                       print authorship information\n\
+    --config=file                 use alternate configuration file\n\
 -T, --type-list                   list available remailers\n\
 -t, --to=user@host                the recipient's address(es)\n\
 -g, --post-to=newsgroup           newsgroup(s) to post to\n\
@@ -481,8 +504,7 @@ Remailer:\n\
 #endif /* USE_SOCK */
 	   "-P, --pop-mail                    force getting messages from POP3 servers\n\
 -G, --generate-key                generate a new remailer key\n\
--K, --update-keys                 generate remailer keys if necessary\n\
-    --config=file                 use alternate configuration file\n"
+-K, --update-keys                 generate remailer keys if necessary\n"
 #ifdef WIN32SERVICE
 	   "\n\
 WinNT service:\n\
@@ -682,7 +704,7 @@ WinNT service:\n\
 
   if (keygen) {
     check_get_pass(0, never_ask_for_passphrase);
-    keymgt(keygen);
+    keymgt(keygen,lifeindays,keysize);
   }
   if (sendpool)
     mix_send();
