@@ -1,4 +1,4 @@
-/* Mixmaster version 3.0  --  (C) 1999 - 2006 Anonymizer Inc. and others.
+/* Mixmaster version 3.1 --  (C) 1999 - 2016 Anonymizer Inc. and others.
 
    Mixmaster may be redistributed and modified under certain conditions.
    This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF
@@ -6,7 +6,7 @@
    details.
 
    Menu-based user interface
-   $Id: menustats.c 934 2006-06-24 13:40:39Z rabbi $ */
+   $Id$ */
 
 
 #if 0
@@ -21,6 +21,12 @@ int menu_getuserpass(BUFFER *p, int mode) { return 0; };
 #include <assert.h>
 #include <stdlib.h>
 
+#ifdef WIN32             // RTC
+#include <io.h>          // RTC
+#else                    // RTC
+#include <unistd.h>      // RTC
+#endif            
+
 #include "menu.h"
 #ifdef WIN32
 #include <urlmon.h>
@@ -40,18 +46,73 @@ int url_download(char *url, char *dest) {
   else
     return 0;
 #else
-  char s[PATHMAX];
-  snprintf(s, PATHMAX, "%s -q %s -O %s", WGET, url, dest);
-  err = system(s);
-
-  if (err != -1) {
-    if (WIFEXITED(err) == 0)
-      return -1; /* abnormal child exit */
-    else
-      return 0;
+  int i;
+  {
+      pid_t pid;
+      int whatprog[2];  /* to indicate outcome of external program to parent process */
+      char tmpwhatprog[10];
+      if (0 != strncmp(url, "http", 4)) {
+          /* url should start "http" */
+          errlog(ERRORMSG, "refusing external web program with url=%s dest=%s\n", url, dest);
+          return 1;
+      }
+      if (pipe(whatprog)) {
+          errlog(ERRORMSG, "pipe failed\n");
+          return 1;
+      }
+      pid=fork();
+      if (!pid) {
+          char *narg[10];
+          close(whatprog[0]);
+          /* arguments for  curl or wget process called with execvp() NOT system()
+             https://www.owasp.org/index.php/Injection_Flaws
+          */
+          narg[0]="curl";
+          narg[1]="-s";
+          narg[2]=url;
+          narg[3]="-o";
+          narg[4]=dest;
+          narg[5]=(char *) 0;
+          execvp("curl", narg);
+	   /* If curl fails use wget as second choice. */
+	  write(whatprog[1], narg[0], 1);
+          narg[0]=WGET;
+          narg[1]="-q";
+          narg[2]=url;
+          narg[3]="-O";
+          narg[4]=dest;
+          narg[5]=(char *) 0;
+          execvp(WGET, narg);
+	  write(whatprog[1], narg[0], 1);
+          exit(1);
+      }
+      close(whatprog[1]);
+      waitpid(pid, &err, 0);
+      i=read(whatprog[0], tmpwhatprog, 3);
+      switch(i) {
+	      case -1:
+                errlog(LOG, "read error on pipe in menustats.c\n");
+		break;
+	      case 0:
+                errlog(LOG, "called external program curl with url=%s dest=%s\n", url, dest);
+		break;
+	      case 1:
+                errlog(LOG, "called external program wget with url=%s dest=%s\n", url, dest);
+		break;
+	      case 2:
+                errlog(LOG, "called no external program to get url=%s\n", url);
+		break;
+	      default:
+                errlog(LOG, "confused in menustats.c\n");
+		break;
+      }
   }
-  else
-    return -1;
+
+  if ((i<=2) && WIFEXITED(err) == 0) {
+      errlog(ERRORMSG, "failure of external program\n");
+      return -1; /* abnormal child exit */
+  } else
+      return 0;
 #endif /* WIN32 */
 }
 
@@ -208,6 +269,9 @@ int stats_download(BUFFER *allpingers, char *sourcename, int curses) {
     mixfile(path, localfiles[i]);
     mixfile(path_t, localfiles[i]);
     strcat(path_t, ".t");
+#ifdef WIN32             // RTC
+    unlink(path);        // RTC
+#endif /* WIN32 */       // RTC
     rename(path_t, path);
   }
   
