@@ -1,4 +1,4 @@
-/* Mixmaster version 3.0  --  (C) 1999 - 2006 Anonymizer Inc. and others.
+/* Mixmaster version 3.1 --  (C) 1999 - 2016 Anonymizer Inc. and others.
 
    Mixmaster may be redistributed and modified under certain conditions.
    This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF
@@ -6,7 +6,7 @@
    details.
 
    Mixmaster initialization, configuration
-   $Id: mix.c 962 2007-11-19 13:42:41Z zax $ */
+   $Id$ */
 
 
 #include "mix3.h"
@@ -57,6 +57,7 @@ char DSAPARAMS[PATHMAX];
 char DHPARAMS[PATHMAX];
 char MIXRAND[PATHMAX];
 char SECRING[PATHMAX];
+char SECRINGNEW[PATHMAX];
 char PUBRING[PATHMAX];
 char IDLOG[PATHMAX];
 char STATS[PATHMAX];
@@ -99,6 +100,7 @@ char ANONADDR[LINELEN];
 char COMPLAINTS[LINELEN];
 int AUTOREPLY;
 char SMTPRELAY[LINELEN];
+int SMTPPORT;
 char SMTPUSERNAME[LINELEN];
 char SMTPPASSWORD[LINELEN];
 
@@ -443,6 +445,7 @@ static void mix_setdefaults()
 	strnncpy(ANONADDR     , "");
 	strnncpy(COMPLAINTS   , "");
 	strnncpy(SMTPRELAY    , "");
+	SMTPPORT              = 25;
 	AUTOREPLY             = 0;
 
 #ifdef USE_SOCK
@@ -488,11 +491,11 @@ static void mix_setdefaults()
 	SENDPOOLTIME  = 0;              /* frequency for sending pool messages */
 	MAILINTIME    = 5 * 60;		/* frequency for processing MAILIN mail */
 
-	KEYLIFETIME      = 13 * 30 * 24 * 60 * 60;	/* validity period for keys. */
-	KEYOVERLAPPERIOD =  1 * 30 * 24 * 60 * 60;	/* when keys have this amount of time */
+	KEYLIFETIME      =  6 * 30 * SECONDSPERDAY;	/* default validity period for keys. */
+	KEYOVERLAPPERIOD =      30 * SECONDSPERDAY;	/* when keys have this amount of time */
 	                                                /* left before expiration, create  */
 	                                        	/* new ones when ./mix -K is run.*/
-	KEYGRACEPERIOD   =       7 * 24 * 60 * 60;	/* accept mail to the old key for this */
+	KEYGRACEPERIOD   =       7 * SECONDSPERDAY;	/* accept mail to the old key for this */
 	                                        	/* amount of time after it has expired. */
 
 
@@ -617,7 +620,8 @@ int mix_configline(char *line)
 
 	  read_conf_i(CLIENTAUTOFLUSH) ||
 	  read_conf_i(MAXRECIPIENTS) ||
-	  
+	  read_conf_i(SMTPPORT) ||
+
 	  read_conf_t(TIMESKEW_FORWARD) ||
 	  read_conf_t(TIMESKEW_BACK) ||
 	  read_conf_i(TEMP_FAIL) );
@@ -653,6 +657,7 @@ int mix_config(void)
   * first match wins.
   *
   *  - what the MIXPATH environment variable points to, if it is set.
+  *  - (MIX3PATH on WIN32 to avoid clashing with earlier versions)
   *  - On WIN32, HKEY_CURRENT_USER\Software\Mixmaster\MixDir, if it exists
   *  - whatever is compiled in with -DSPOOL
   *  - On Win32 %APPDATA%\Mixmaster
@@ -660,8 +665,14 @@ int mix_config(void)
   *  - the current working directory
   */
 
-  if (err == -1 && (d = getenv("MIXPATH")) != NULL)
-    err = mixdir(d, 1);
+  if (err == -1 && (d = getenv(
+#ifdef WIN32
+    // MIX3PATH comes from RTC
+    "MIX3PATH"
+#else
+    "MIXPATH"
+#endif /* WIN32 */
+                )) != NULL) err = mixdir(d, 1);
 
 #ifdef WIN32
   RegOpenKeyEx(HKEY_CURRENT_USER, "Software", 0, KEY_ALL_ACCESS, &regsw);
@@ -730,6 +741,9 @@ int mix_config(void)
     fclose(f);
   }
 
+  /* set SECRINGNEW after reading SECRING from cfg file */
+  strncpy(SECRINGNEW, SECRING, PATHMAX);
+  strncat(SECRINGNEW, ".new", PATHMAX); SECRINGNEW[PATHMAX-1]='\0';
   mixfile(POOLDIR, POOL); /* set POOLDIR after reading POOL from cfg file */
   if (POOLDIR[strlen(POOLDIR) - 1] == DIRSEP)
     POOLDIR[strlen(POOLDIR) - 1] = '\0';
@@ -904,8 +918,31 @@ void mix_check_timeskew() {
   }
 }
 
+#ifdef WIN32                             // RTC
+char* mix_version(void)                  // RTC
+{                                        // RTC
+  return VERSION;                        // RTC
+}                                        // RTC
+int mix_initex(char *mixdir)             // RTC
+{                                        // RTC
+  return mix_init(mixdir);               // RTC
+}                                        // RTC
+#endif /* WIN32 */                       // RTC
+
 int mix_init(char *mixdir)
 {
+#ifdef WIN32                             // RTC
+  char mixpath[PATHMAX + 50];             // RTC
+  if (mixdir)                            // RTC
+  {                                      // RTC
+     strcpy(mixpath, "MIX3PATH=");       // RTC
+     strncat(mixpath,  mixdir, PATHMAX); // RTC
+     putenv(mixpath);                    // RTC
+     initialized = 0;                    // RTC
+     rnd_state = RND_WILLSEED;           // RTC
+  }                                      // RTC
+#endif /* WIN32 */                       // RTC
+
   if (!initialized) {
     if (mixdir)
       strncpy(MIXDIR, mixdir, LINELEN);
@@ -1022,7 +1059,7 @@ int mix_daily(void)
   pgpmaxexp();
   pool_packetexp();
   stats(NULL);
-  keymgt(0);
+  keymgt(0,0,4096);
   return (0);
 }
 
@@ -1054,7 +1091,7 @@ void sighandler(int signal) {
 
     @param restart  Whether or not system calls should be
                     restarted. Usually we want this, the
-                    only excetion is the sleep() in the
+                    only exception is the sleep() in the
                     daemon mail loop.
     @author         PP
     @return         -1 if calling sigaction failed, 0 on
@@ -1201,7 +1238,7 @@ void errlog(int type, char *fmt,...)
   msg = buf_new();
   buf_appends(msg, line);
   p = msg->length;
-  buf_appendf(msg, "%s: ", err[type]);
+  buf_appendf(msg, "%s: [%d] ", err[type], getpid());
   va_start(args, fmt);
   buf_vappendf(msg, fmt, args);
   va_end(args);
