@@ -1,4 +1,4 @@
-/* Mixmaster version 3.0  --  (C) 1999 - 2006 Anonymizer Inc. and others.
+/* Mixmaster version 3.1 --  (C) 1999 - 2016 Anonymizer Inc. and others.
 
    Mixmaster may be redistributed and modified under certain conditions.
    This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF
@@ -802,6 +802,10 @@ int buf_unzip(BUFFER *buf, int type);
 #ifndef _MIXLIB_H
 #define _MIXLIB_H
 
+#define RSASTATSFILE "rsastats.daily"
+#define RSATEXTFILE  "rsastats.txt"
+int mix_global_verbose;
+
 #include <stdio.h>
 #include <time.h>
 #ifdef WIN32
@@ -843,6 +847,12 @@ int mix_encrypt(int type, BUFFER *message, char *chain, int numcopies,
 int mix_decrypt(BUFFER *message);
 int mix_send(void);
 
+int derive_aes_keys(BUFFER *aes_pre_key, BUFFER *hkey, BUFFER *aes_header_key, BUFFER *aes_body_key, BUFFER *aes_tte_key, BUFFER *aes_iv, BUFFER *aes_body_iv, BUFFER *aes_header_iv);
+int hmac_sha256(BUFFER *b, BUFFER *hk, BUFFER *md);
+int digest_sha256(BUFFER *b, BUFFER *md);
+char *showdata(BUFFER *buf, int max);
+int write_pubkey_file(FILE *in, FILE *out, BUFFER *pk_found, byte *k1_found, time_t created_found, time_t expires_found);
+
 #define FORCE_POOL 1
 #define FORCE_POP3 2
 #define FORCE_DAILY 4
@@ -915,3 +925,4 @@ time_t parsedate(char *p);
 #endif /* WIN32 */
 
 #endif /* not _MIXLIB_H */
+
