@@ -1,4 +1,4 @@
-/* Mixmaster version 3.0  --  (C) 1999 - 2006 Anonymizer Inc. and others.
+/* Mixmaster version 3.1 --  (C) 1999 - 2016 Anonymizer Inc. and others.
 
    Mixmaster may be redistributed and modified under certain conditions.
    This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF
@@ -6,7 +6,7 @@
    details.
 
    Function prototypes
-   $Id: mix3.h 934 2006-06-24 13:40:39Z rabbi $ */
+   $Id$ */
 
 
 #ifndef _MIX3_H
@@ -210,7 +210,7 @@ int digest_sha1(BUFFER *b, BUFFER *md);
 int digest_rmd160(BUFFER *b, BUFFER *md);
 
 #define KEY_ID_LEN 32
-int keymgt(int force);
+int keymgt(int force,long int lifeindays,long int keysize);
 int key(BUFFER *b);
 int adminkey(BUFFER *b);
 
@@ -226,6 +226,7 @@ int buf_3descrypt(BUFFER *b, BUFFER *key, BUFFER *iv, int enc);
 int buf_castcrypt(BUFFER *b, BUFFER *key, BUFFER *iv, int enc);
 #ifdef USE_AES
 int buf_aescrypt(BUFFER *b, BUFFER *key, BUFFER *iv, int enc);
+int buf_aes_ctr128(BUFFER *buf, BUFFER *key, BUFFER *iv);
 #endif /* USE_AES */
 
 int db_getseckey(byte keyid[], BUFFER *key);
@@ -234,7 +235,7 @@ int pk_decrypt(BUFFER *encrypted, BUFFER *privkey);
 int pk_encrypt(BUFFER *plaintext, BUFFER *privkey);
 int check_seckey(BUFFER *buf, const byte id[]);
 int check_pubkey(BUFFER *buf, const byte id[]);
-int v2createkey(void);
+int v2createkey(long int lifeindays,long int keysize);
 int getv2seckey(byte keyid[], BUFFER *key);
 int seckeytopub(BUFFER *pub, BUFFER *sec, byte keyid[]);
 
@@ -244,6 +245,11 @@ int mix_config(void);
 int mix_initialized(void);
 int mix_daily(void);
 
+#ifdef WIN32                  // RTC
+char* mix_version(void);      // RTC
+int mix_initex(char *mixdir); // RTC
+#endif                        // RTC
+
 /* message pool */
 #define INTERMEDIATE 0
 int pool_send(void);
@@ -308,6 +314,9 @@ typedef struct {
   int version;
   char addr[128];
   byte keyid[16];
+  time_t expires;
+  int rsalen;
+  int use_cfb;  /* CFB mode used with AES - being replaced by CTR mode */
   struct {
     unsigned int mix:1;
     unsigned int compress:1;
@@ -319,6 +328,7 @@ typedef struct {
     unsigned int hash:1;
     unsigned int ek:1;
     unsigned int esub:1;
+    unsigned int hsub:1;
 
     unsigned int nym:1;
     unsigned int newnym:1;
@@ -370,6 +380,7 @@ int v2partial(BUFFER *body, BUFFER *mid, int packet, int numpackets);
 int v2_merge(BUFFER *mid);
 int mix_armor(BUFFER *in);
 int mix_dearmor(BUFFER *armored, BUFFER *bin);
+int deleteoldkeys(void);
 
 /* type 1 */
 #define HDRMARK "::"
