@@ -1,4 +1,4 @@
-/* Mixmaster version 3.0  --  (C) 1999 - 2006 Anonymizer Inc. and others.
+/* Mixmaster version 3.1 --  (C) 1999 - 2016 Anonymizer Inc. and others.
 
    Mixmaster may be redistributed and modified under certain conditions.
    This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF
@@ -6,7 +6,7 @@
    details.
 
    OpenPGP messages
-   $Id: pgp.h 934 2006-06-24 13:40:39Z rabbi $ */
+   $Id$ */
 
 
 #ifdef USE_PGP
@@ -174,7 +174,7 @@ typedef struct {
   int type; /* undefined, public, private */
   char filename[LINELEN];
   BUFFER *encryptkey;
-#ifndef NDEBUG
+#ifdef DEBUG
   int writer;
 #endif
 } KEYRING;
