@@ -1,4 +1,4 @@
-/* Mixmaster version 3.0  --  (C) 1999 - 2006 Anonymizer Inc. and others.
+/* Mixmaster version 3.1 --  (C) 1999 - 2016 Anonymizer Inc. and others.
 
    Mixmaster may be redistributed and modified under certain conditions.
    This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF
@@ -6,7 +6,7 @@
    details.
 
    Send messages from pool
-   $Id: pool.c 934 2006-06-24 13:40:39Z rabbi $ */
+   $Id$ */
 
 #include "mix3.h"
 #include <stdlib.h>
@@ -509,6 +509,11 @@ int msg_send(char *name)
       err = filtermsg(m);
       if (err == 1)
 	userfrom = 1, err = 0;
+      if (err == 2) {
+        errlog(LOG, "All recipients filtered out. Rejecting message\n");
+        err = -1;
+        goto end;
+      }
       if (err != -1) {
 	/* message has recipients */
 	errlog(DEBUGINFO, "Sending message (%ld bytes)\n", m->length);
@@ -617,22 +622,42 @@ int allowmessage(BUFFER *in)
 
 int doallow(BUFFER *line, BUFFER *filter)
 /* line is a To, CC or BCC line.
- * problem is: there may be multiple addresses in one header
- * line but we only want to allow if _all_ are allowed
  *
- * So to not send direct if we do not want, we _never_ send
- * direct if there is more than one address: This is
- * assumed to be the case when there is a
- * comma in the header line.
- *
- * this should probably be rewritten somehwhen. therefore: FIXME
+ * there may be multiple addresses in one header
+ * line but we only allow if _all_ are allowed
  *
  * returns: 1 if allowed
  *          0 if message should be send indirectly
  */
 {
-  if (strchr( line->data, ',')) return 0;
-  return doblock(line, filter, 0);
+  BUFFER *newlinelist;
+  BUFFER *addrs;
+  int res = 1;
+
+  newlinelist = buf_new();
+  addrs = buf_new();
+
+  while (buf_getc(line) != ':');	/* find the : */
+  buf_getc(line);			/* skip the space */
+  buf_rest(addrs, line);		/* and copy the addresses */
+  buf_rewind(line);
+
+  rfc822_addr(addrs, newlinelist);	/* split into one address per line */
+
+  buf_clear(addrs);
+
+  while (buf_getline(newlinelist, addrs) != -1) {
+    if (doblock(addrs, filter, 0) == 0) {
+      res = 0;
+      errlog(DEBUGINFO, "Destination %b not allowed.\n", addrs);
+    }
+    buf_clear(addrs);
+  }
+
+  buf_free(newlinelist);
+  buf_free(addrs);
+
+  return (res);
 }
 
 int filtermsg(BUFFER *in)
@@ -784,8 +809,9 @@ int filtermsg(BUFFER *in)
 
   /* return 1 for user supplied From line */
   err = from;
+  // Return 2 if no valid recipients in the message
   if (dest == 0)
-    err = -1;
+    err = 2;
 
 end:
   buf_move(in, out);
@@ -834,10 +860,10 @@ BUFFER *readdestblk( )
 }
 
 int doblock(BUFFER *line, BUFFER *filter, int logandreset)
-/* logandreset is usually 0
- * it is only set to 1 when called from doallow
- *  which only checks whether messages are allowed to
- *  be sent directly
+/* logandreset is usually 1
+ * it is only set to 0 when called from doallow
+ * which only checks whether messages are allowed to
+ * be sent directly
  */
 {
   int block = 0;
