@@ -1,4 +1,4 @@
-/* Mixmaster version 2.9  --  (C) 1999 - 2003 Anonymizer Inc. and others.
+/* Mixmaster version 3.1 --  (C) 1999 - 2016 Anonymizer Inc. and others.
 
    Mixmaster may be redistributed and modified under certain conditions.
    This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF
@@ -6,7 +6,7 @@
    details.
 
    Get randomness from device or user
-   $Id: rndseed.c 665 2003-11-09 01:47:32Z rabbi $ */
+   $Id$ */
 
 
 #include "mix3.h"
@@ -96,7 +96,7 @@ int rnd_seed(void)
     if (!isatty(fileno(stdin)))
       rnd_error();
 #else /* end of 1 */
-#error "should initialize the prng from system ressources"
+#error "should initialize the prng from system resources"
 #endif /* else if not 1 */
     fprintf(stderr, "Please enter some random characters.\n");
     kbd_noecho();
@@ -116,11 +116,7 @@ int rnd_seed(void)
 	}
     }
     fprintf(stderr, "Thanks.\n");
-#ifdef WIN32
-    Sleep(1000);
-#else /* end of WIN32 */
     sleep(1);
-#endif /* else if not WIN32 */
     kbd_echo();
   }
 #ifdef DEV_RANDOM
@@ -149,17 +145,13 @@ int rnd_seed(void)
       }
       if (isatty(fileno(stdin))) {
 	fprintf(stderr, "Thanks.\n");
-#ifdef WIN32
-	sleep(1000);
-#else
 	sleep(1);
-#endif
 	kbd_echo();
       }
       close(fd);
     }
   }
-#endif
+#endif /* DEV_RANDOM */
   rnd_state = RND_SEEDED;
   return (0);
 }
