@@ -1,4 +1,4 @@
-/* Mixmaster version 3.0  --  (C) 1999 - 2006 Anonymizer Inc. and others.
+/* Mixmaster version 3.1 --  (C) 1999 - 2016 Anonymizer Inc. and others.
 
    Mixmaster may be redistributed and modified under certain conditions.
    This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF
@@ -6,7 +6,7 @@
    details.
 
    Remailer statistics
-   $Id: stats.c 934 2006-06-24 13:40:39Z rabbi $ */
+   $Id$ */
 
 
 #include "mix3.h"
@@ -54,7 +54,7 @@ int stats_out(int pool)
 
 int stats(BUFFER *b)
 {
-  FILE *s, *f;
+  FILE *s, *f, *rs;
   char line[LINELEN];
   long now, today, then;
   time_t t;
@@ -239,6 +239,19 @@ int stats(BUFFER *b)
       buf_nl(b);
     }
   }
+  if (b) {
+    if ((rs=mix_openfile(RSATEXTFILE, "r")) != NULL) {
+      buf_nl(b);
+      buf_appends(b, "Mix key sizes used (1024, 2048, 3072 and 4096 bit RSA)\n");
+      while ( fgets(line, sizeof(line), rs) ) {
+          line[sizeof(line)-1]='\0';
+          buf_append(b, line, strlen(line));
+          if (line[strlen(line)-1] != '\n') buf_nl(b);
+      }
+      buf_nl(b);
+      fclose(rs);
+    }
+  }
   return (0);
 }
 
@@ -408,17 +421,20 @@ void conf_premail(BUFFER *out)
     buf_appends(out, " remix");
   else if (REMIX)
     buf_appends(out, " remix2");
-  if (PGP || UNENCRYPTED)
-    buf_appends(out, " latent hash cut test");
+  if (PGP || UNENCRYPTED) {
+#ifdef USE_IDEA
+    buf_appends(out, " esub");
+#else
+    buf_appends(out, " esubbf");
+#endif /* USE_IDEA */
+    buf_appends(out, " hsub latent hash cut test");
+  }
   if (PGP) {
 #ifdef USE_IDEA
     buf_appends(out, " ek");
 #endif /* USE_IDEA */
     buf_appends(out, " ekx");
   }
-#ifdef USE_IDEA
-  buf_appends(out, " esub");
-#endif /* USE_IDEA */
 #if 0				/* obsolete */
 #ifdef USE_NSUB
   buf_appends(out, " nsub");
